/**
 * Demo of the butterfly renderer
 * 
 * Copyright (c) 2008 Tom Beddard
 * http://www.subblue.com
 * 
 * Licensed under the MIT License: http://www.opensource.org/licenses/mit-license.php
 * 
 * Credits
 * Clifford Pickover: Computers and the Imagination ISBN: 0-312-06131-5
 */

package
{
	import flash.display.Sprite;
	import flash.display.Shape;
	import flash.display.Stage;
	import flash.display.StageAlign;
	import flash.display.StageQuality;
	import flash.display.StageScaleMode;
	import flash.display.GradientType;
	import flash.display.BitmapData;
	import flash.display.LineScaleMode;
	import flash.geom.Matrix;
	import flash.geom.Rectangle;
	import fl.controls.Slider;
	import fl.controls.Label;
	import fl.controls.Button;
	import fl.controls.ColorPicker;
	import fl.events.ComponentEvent;
	import fl.events.SliderEvent;
	import fl.events.ColorPickerEvent;
	import fl.events.ComponentEvent;
	import flash.text.TextFieldAutoSize;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.events.KeyboardEvent;
	import flash.events.ContextMenuEvent;
	import flash.ui.ContextMenu;
	import flash.ui.ContextMenuItem;
	import flash.utils.*;
	import flash.printing.PrintJob;
	import flash.printing.PrintJobOrientation;
	import com.subblue.butterfly.Butterfly;
	import com.subblue.utils.*;
	
	
	public class Main extends Sprite
	{
		private var page:Sprite;
		private var graph:Butterfly;
		private var bg:Shape;
		private var border:Shape;
		
		private var amplitude:Slider 	= new Slider();
		private var amplitudeLabel:Label = new Label();
		
		private var steps:Slider 		= new Slider();
		private var stepsLabel:Label 	= new Label();
		private var rotationSlider:Slider = new Slider();
		private var rotationLabel:Label   = new Label();
		private var verticalSlider:Slider = new Slider();
		private var verticalLabel:Label   = new Label();
		
		private var aSlider:Slider 		= new Slider();
		private var aLabel:Label 		= new Label();
		private var bSlider:Slider 		= new Slider();
		private var bLabel:Label 		= new Label();
		private var cSlider:Slider 		= new Slider();
		private var cLabel:Label 		= new Label();
		private var ASlider:Slider 		= new Slider();
		private var ALabel:Label 		= new Label();
		private var BSlider:Slider 		= new Slider();
		private var BLabel:Label 		= new Label();
		
		private var opacity:Slider 		= new Slider();
		private var opacityLabel:Label 	= new Label();
		private var iterations:Slider 	= new Slider();
		private var iterationsLabel:Label 	= new Label();
		private var thickness:Slider 	= new Slider();
		private var thicknessLabel:Label 	= new Label();
		
		private var colorPicker1:ColorPicker = new ColorPicker();
		private var colorPicker2:ColorPicker = new ColorPicker();
		private var colorPicker3:ColorPicker = new ColorPicker();
		
		private var printButton:Button;
		private var exportUrl:String = "/export";
		private var exportExt:String = "jpg";
		private var exportQuality:int = 90;
		
		private var gui:Sprite;
		private var options = [
			{name: 'Scale', slider:amplitude, min:0.1, max:2, value: 1, snap:0.1,  ticks:0.5, label:amplitudeLabel},
			{name: 'Revolutions', slider:steps, min:10, max:200, value: 100, snap:10,   ticks:10, label:stepsLabel},
			{name: 'Rotation', slider:rotationSlider, min:0, max:180, value: 90, snap:1, ticks:36, label:rotationLabel},
			{name: 'Vertical', slider:verticalSlider, min:-100, max:100, value: 0, snap:1, ticks:20, label:verticalLabel},
			{name: 'a', slider:aSlider, min:0.1, max:20, value: 1, snap:0.1, ticks:10, label:aLabel},
			{name: 'b', slider:bSlider, min:0.1, max:20, value: 4, snap:0.1, ticks:10, label:bLabel},
			{name: 'c', slider:cSlider, min:0.1, max:100, value: 30, snap:0.1, ticks:10, label:cLabel},
			{name: 'A', slider:ASlider, min:-10, max:10, value: 2, snap:0.1, ticks:10, label:ALabel},
			{name: 'Power', slider:BSlider, min:1, max:10, value: 5, snap:1, ticks:10, label:BLabel},
			{name: 'Opacity', slider:opacity, min:0.01, max:1, value: 0.25, snap:0.01, ticks:0.1, label:opacityLabel},
			{name: 'Thickness', slider:thickness, min:1, max:50, value: 1, snap:1, ticks:5, label:thicknessLabel}
			];
		
		
		public function Main()
		{
			stage.scaleMode = StageScaleMode.NO_SCALE;
			stage.align = StageAlign.TOP_LEFT;
			stage.showDefaultContextMenu = true;
			stage.quality = StageQuality.BEST;
			init();
		}
		
		
		private function init(e:Event = null):void
		{
			if (stage.stageWidth)
			{
				page = new Sprite();
				bg = new Shape();
				bg.graphics.beginFill(0xFFFFFF, 1);
				bg.graphics.drawRect(0, 0, stage.stageWidth, stage.stageHeight);
				bg.graphics.endFill();
				page.addChild(bg);
				
				graph = new Butterfly();
				page.addChild(graph);
				addChild(page);
				
				border = new Shape();
				border.graphics.lineStyle(1, 0xCCCCCC, 1, true, LineScaleMode.NONE);
				border.graphics.drawRect(0.5, 0.5, stage.stageWidth - 0.5, stage.stageHeight - 0.5);
				addChild(border);
				
				positionUI();
				
				var customMenu:ContextMenu = new ContextMenu();
				customMenu.hideBuiltInItems();
				
				var contextMenuItem:ContextMenuItem = new ContextMenuItem("Print Guilloche");
				contextMenuItem.addEventListener(ContextMenuEvent.MENU_ITEM_SELECT, printPage);
				
				customMenu.customItems.push(contextMenuItem);
				this.contextMenu = customMenu;
				
				setupUI();
				
				stage.addEventListener("resize", positionUI);
				stage.addEventListener(KeyboardEvent.KEY_UP, keyListener);
				setTimeout(updateUI, 300);
				
			} else {
				setTimeout(init, 100);
			}
		}
		
		
		public function exportImage(e:Event = null):void
		{
			var render:BitmapData = new BitmapData(bg.width, bg.height, false, 0xFFFFFFFF);
			render.draw(page);
			
			var imageExporter:ImageExporter = new ImageExporter(exportUrl, render, exportExt, exportQuality);
			imageExporter.export("butterfly_" + imageExporter.timestamp());
		}
		
		
		private function printPage(e:ContextMenuEvent = null):void
		{
			var printJob:PrintJob = new PrintJob();
			
			if (printJob.start()) {
				try {
			  		printJob.addPage(page, new Rectangle(0, 0, bg.width, bg.height));
				}
				catch(e:Error) {
			  		// handle error
				}
				
				printJob.send();
			}
		}
		
		
		private function positionUI(e:Event = null):void
		{
			graph.x = (stage.stageWidth / 2);
			graph.y = stage.stageHeight / 2;
			bg.width = stage.stageWidth;
			bg.height = stage.stageHeight;
			border.width = stage.stageWidth - 0.5;
			border.height = stage.stageHeight - 0.5;
		}
		
		
		private function setupUI():void
		{
			var label:Label;
			gui = new Sprite();
			addChild(gui);
			gui.x = 15;
			gui.y = 15;
			
			var guiBg = new Shape();
			guiBg.graphics.beginFill(0x999999, 0.2);
			guiBg.graphics.drawRoundRect(0, 0, 240, 350, 10, 10);
			guiBg.graphics.endFill();
			guiBg.x = -10;
			guiBg.y = -10;
			gui.addChild(guiBg);
			
			for (var i:int = 0; i < options.length; i++)
			{
				label = new Label();
				label.text = options[i].name;
				label.move(0, i * 20);
				label.autoSize = TextFieldAutoSize.RIGHT;
				label.width = 70;
				gui.addChild(label);
				
				options[i].slider.liveDragging = true;
				options[i].slider.minimum = options[i].min;
				options[i].slider.maximum = options[i].max;
				options[i].slider.snapInterval = options[i].snap;
				options[i].slider.tickInterval = options[i].ticks;
				options[i].slider.value = options[i].value;
				options[i].slider.width = 100;
				options[i].slider.move(80, (i * 20) + 5);
				options[i].slider.addEventListener(SliderEvent.CHANGE, updateUI);
				options[i].slider.addEventListener(KeyboardEvent.KEY_DOWN, fineAdjustmentListener);
				options[i].slider.addEventListener(MouseEvent.ROLL_OVER, enableFocus);
				gui.addChild(options[i].slider);
				
				options[i].label.text = String(options[i].slider.value);
				options[i].label.move(190, i * 20);
				gui.addChild(options[i].label);
			}
			
			var colorLabel1:Label = new Label();
			colorLabel1.text = 'Line colours';
			colorLabel1.move(0, (options.length * 20) + 10);
			colorLabel1.autoSize = TextFieldAutoSize.RIGHT;
			colorLabel1.width = 70;
			gui.addChild(colorLabel1);
			
			colorPicker1.selectedColor = 0x000000;
			colorPicker1.move(80, (options.length * 20) + 10);
			gui.addChild(colorPicker1);
			
			colorPicker2.selectedColor = 0x000000;
			colorPicker2.move(120, (options.length * 20) + 10);
			gui.addChild(colorPicker2);
			
			var colorLabel2:Label = new Label();
			colorLabel2.text = 'Background';
			colorLabel2.move(0, (options.length + 2) * 20);
			colorLabel2.autoSize = TextFieldAutoSize.RIGHT;
			colorLabel2.width = 70;
			gui.addChild(colorLabel2);
			
			colorPicker3.selectedColor = 0xFFFFFF;
			colorPicker3.move(80, (options.length + 2) * 20);
			gui.addChild(colorPicker3);
			
			colorPicker1.addEventListener(ColorPickerEvent.CHANGE, changeColor);
			colorPicker2.addEventListener(ColorPickerEvent.CHANGE, changeColor);
			colorPicker3.addEventListener(ColorPickerEvent.CHANGE, changeColor);
			
			var exportButton:Button = new Button();
			exportButton.label = 'Save image';
			exportButton.move(80, (options.length * 27) + 5);
			exportButton.addEventListener(ComponentEvent.BUTTON_DOWN, exportImage);
			gui.addChild(exportButton);
			
		}
		
		
		private function enableFocus(e:MouseEvent):void
		{
			if (e.currentTarget is Slider)
				e.currentTarget.setFocus();
		}
		
		
		
		private function fineAdjustmentListener(e:KeyboardEvent):void
		{
			switch (e.keyCode)
			{
				case 38: // Up arrow
				case 39: // right arrow
				minorIncrement(e.currentTarget, 1);
				updateUI();
				break;
				
				case 40: // Down arrow
				case 37: // Left arrow
				minorIncrement(e.currentTarget, -1);
				updateUI();
				break;
			}
		}
		
		private function keyListener(e:KeyboardEvent):void
		{
			switch (e.keyCode)
			{
				case 72: // h key
				gui.visible = gui.visible ? false : true;
				break;
				
				case 83: // s key
				exportImage();
			}
		}
		
		
		private function minorIncrement(s:*, i:int = 1):void
		{
			var d:Number = 0;
			var si:Number;
			
			d = Number(s.snapInterval) / 10;
			si = s.snapInterval;
			s.snapInterval = 0.000001;
			s.value += (d * i);
			s.snapInterval = si;
		}
		
		
		private function setBg():void
		{
			bg.graphics.beginFill(colorPicker3.selectedColor, 1);
			bg.graphics.drawRect(0, 0, stage.stageWidth, stage.stageHeight);
			bg.graphics.endFill();
		}
		
		
		private function changeColor(e:Event):void
		{
			if (colorPicker1.selectedColor == colorPicker2.selectedColor && colorPicker3.selectedColor == colorPicker1.selectedColor)
			{
				if (colorPicker3.selectedColor == 0)
				{
					colorPicker1.selectedColor = colorPicker2.selectedColor = 0xFFFFFF;
				}
				else if (colorPicker3.selectedColor == 0xFFFFFF)
				{
					colorPicker1.selectedColor = colorPicker2.selectedColor = 0x000000;
				}
			}
			setBg();
			updateUI();
		}
		
		
		private function updateUI(e:Event = null):void
		{
			amplitudeLabel.text = String(amplitude.value);
			stepsLabel.text = String(steps.value);
			rotationLabel.text = String(rotationSlider.value);
			verticalLabel.text = String(verticalSlider.value);
			aLabel.text = String(aSlider.value);
			bLabel.text = String(bSlider.value);
			cLabel.text = String(cSlider.value);
			ALabel.text = String(ASlider.value);
			BLabel.text = String(BSlider.value);
			opacityLabel.text = String(opacity.value);
			thicknessLabel.text = String(thickness.value);
			
			
			stage.quality = StageQuality.LOW;
			// Set properties
			graph.amplitude = amplitude.value;
			graph.revolutions = steps.value;
			graph.rotationOffset = rotationSlider.value;
			graph.verticalOffset = verticalSlider.value;
			graph.a = aSlider.value;
			graph.b = bSlider.value;
			graph.c = cSlider.value;
			graph.A = ASlider.value;
			graph.B = BSlider.value;
			graph.thickness = thickness.value;
			graph.opacity = opacity.value;
			graph.color1 = colorPicker1.selectedColor;
			graph.color2 = colorPicker2.selectedColor;
			graph.plot();
			stage.quality = StageQuality.BEST;
		}
	}
}