/**
 * Butterfly
 * 
 * Copyright (c) 2008 Tom Beddard
 * http://www.subblue.com
 * 
 * Licensed under the MIT License: http://www.opensource.org/licenses/mit-license.php
 * 
 * Credits
 * Clifford Pickover: Computers and the Imagination ISBN: 0-312-06131-5
 */

package com.subblue.butterfly
{
	import flash.display.Sprite;
	import flash.display.Shape;
	import com.subblue.utils.ColorUtil;
	
	public class Butterfly extends Sprite
	{
		private var thetaStep:Number	= 0.01;			// Angle increment
		private var _steps:int			= 1000;
		private var _revolutions:int 	= 100;			// The number of times round
		private var _a:Number			= 1;			// The first theta multiplier
		private var _b:Number			= 4;			// The second theta multiplier
		private var _c:Number			= 12;			// The third theta divisor
		private var _A:Number			= 2;			// Cosine multiplier
		private var _B:Number			= 5;			// Sine power

		private var _rotation:Number 	= 0;			// The rotation offset
		private var _yOffset:Number 	= 0;			// The y-offset
		
		private var _r:Number 			= 0.25;			// The minor ripple
		private var _p:Number 			= 25;			// Radius type effect
		private var _m:Number			= 1;			// Angle multiplier
		private var _amplitude:Number 	= 50;			// Scale of final drawing
		
		private var _opacity:Number 	= 0.2;			// Line opacity
		private var _color1:uint 		= 0x000000;		// Line start colour
		private var _color2:uint 		= 0x000000;		// Line end colour
		private var _thickness:Number 	= 1;			// Line thickness
		
		private var section_length:int 	= 10;			// Number of sections for each line
		private var deg2rad:Number 		= Math.PI/180;	// Factor to convert degs to radians
		private var _colorCycle:int 	= 1;			// The number of times the colour repeats
		private var colorPalette:Array	= [];			// For storing a gradient array of colours
		
		public function Butterfly()
		{
	    	super();
		}
		
		// r = exp^cos(a * theta) - Acos(b * theta) + sin^B(theta/c)
    	// theta = 0..2PI
		// R = 50, r = -0.25, p=25
		public function plot():void
		{
			if (numChildren > 0) removeChildAt(0);
			
			var plot:Sprite = new Sprite();
			var l:Shape;
			var x:Number, y:Number, ox:Number, oy:Number, r:Number;
			var sl:int = 0;
			var theta:Number = 0;
			_steps = _revolutions * Math.PI / thetaStep;
			
			setColorPalette();
			var cpl:int = colorPalette.length;
			var c:uint = _color1;
			
			for (var t:int = 0; t <= _steps; t++)
			{
				r = (Math.exp(Math.cos(_a * theta)) - _A * Math.cos(_b * theta) + Math.pow(Math.sin(theta / _c), _B)) * _amplitude;
				
				x = r * Math.cos(theta - _rotation);
				y = r * Math.sin(theta - _rotation) + _yOffset;
				
				if (sl == 0)
				{
					c = (_color1 == _color2 || cpl < 2) ? _color1 : colorPalette[t % cpl];
					// Create new line section
					l = new Shape();
					l.graphics.lineStyle(_thickness, c, _opacity, false, "normal", "none", "round");
					
					if (t == 0)
					{
						l.graphics.moveTo(x, y);
					} else {
						l.graphics.moveTo(ox, oy);
						l.graphics.lineTo(x, y);
					}
					plot.addChild(l);
				} else {
					// Append to line section
					l.graphics.lineTo(x, y);
				}
				
				ox = x;
				oy = y;
				sl++;
				theta += thetaStep;
				
				if (sl == section_length) sl = 0;
			}
			addChild(plot);
		}
		
		
		private function setColorPalette():void
		{
			colorPalette = ColorUtil.blendArray(_color1, _color2, ((_steps / section_length) - 1), false)
			colorPalette = colorPalette.concat(ColorUtil.blendArray(_color2, _color1, ((_steps / section_length) - 1), false))
		}
		
		
		/**
		 *	Get and set amplitude
		 */
		public function get amplitude():Number { 
			return _amplitude / 50; 
		}
		public function set amplitude(value:Number):void {
			_amplitude = value * 50;
		}
		
		
		/**
		 *	Set colour cycle steps
		 */
		public function get colorCycle():int { 
			return _colorCycle; 
		}
		public function set colorCycle(value:int):void {
			if (value !== _colorCycle) 
			{
				_colorCycle = value;
				setColorPalette();
			}
		}
		
		/**
		 *	Get and set revolution value - the number of times we spin the butterfly
		 */
		public function get revolutions():int { 
			return _revolutions; 
		}
		public function set revolutions(value:int):void {
			if (value !== _revolutions) _revolutions = value;
		}
		
		/**
		 *	Get and set rotation offset
		 */
		public function get rotationOffset():Number { 
			return _rotation; 
		}
		public function set rotationOffset(value:Number):void {
			_rotation = value * deg2rad;
		}
		
		/**
		 *	Get and set vertical offset
		 */
		public function get verticalOffset():Number { 
			return _yOffset; 
		}
		public function set verticalOffset(value:Number):void {
			_yOffset = value;
		}
		
		/**
		 *	Get and set a value
		 */
		public function get a():Number { 
			return _a; 
		}
		public function set a(value:Number):void {
			if (value !== _a) _a = value;
		}
		
		/**
		 *	Get and set b value
		 */
		public function get b():Number { 
			return _b; 
		}
		public function set b(value:Number):void {
			if (value !== _b) _b = value;
		}
		
		/**
		 *	Get and set c value
		 */
		public function get c():Number { 
			return _c; 
		}
		public function set c(value:Number):void {
			if (value !== _c) _c = value;
		}
		
		/**
		 *	Get and set A value
		 */
		public function get A():Number { 
			return _A; 
		}
		public function set A(value:Number):void {
			if (value !== _A) _A = value;
		}
		
		/**
		 *	Get and set B value
		 */
		public function get B():Number { 
			return _B; 
		}
		public function set B(value:Number):void {
			if (value !== _B) _B = value;
		}
		
		/**
		 *	Get and set opacity
		 */
		public function get opacity():Number { 
			return _opacity; 
		}
		public function set opacity(value:Number):void {
			if (value !== _opacity) _opacity = value;
		}
		
		/**
		 *	Get and set thickness
		 */
		public function get thickness():Number { 
			return _thickness; 
		}
		public function set thickness(value:Number):void {
			if (value !== _thickness) _thickness = value;
		}
		
		/**
		 *	Get and set colour
		 */
		public function get color1():uint { 
			return _color1; 
		}
		public function set color1(value:uint):void { 
			if (value !== _color1)
			{
				_color1 = value;
				setColorPalette();
			}
		}
		public function get color2():uint { 
			return _color1; 
		}
		public function set color2(value:uint):void { 
			if (value !== _color2)
			{
				_color2 = value;
				setColorPalette();
			}
		}
	}
}