/**
 * Image Exporter
 * 
 * @author		Tom Beddard
 * @version 	0.1
 * 
 * Licensed under the MIT License: http://www.opensource.org/licenses/mit-license.php
 * Uses the adobe PNGEncoder and JPGEncoder classes
 */

package com.subblue.utils
{
	import com.adobe.images.*;
	import flash.display.BitmapData;
	import flash.net.URLRequest;
	import flash.net.URLRequestMethod;
	import flash.net.URLRequestHeader;
	import flash.net.URLLoaderDataFormat;
	import flash.net.navigateToURL;
	import flash.utils.ByteArray;
	
	public class ImageExporter
	{
		private var _url:String;
		private var _bitmapData:BitmapData;
		private var _extension:String = 'png';
		private var _quality:int = 70;
		private var _filename:String = '';
		
		public function ImageExporter(u:String = null, b:BitmapData = null, e:String = null, q:int = 70)
		{
			if (u) _url = u;
			if (b) _bitmapData = b;
			if (e) _extension = e;
			_quality = q;
		}
		
		public function export(filename:String = ''):void
		{
			var img:ByteArray;
			
			if (filename == '')
				filename = timestamp();
			
			_filename = filename;
			
			if (_extension == 'png')
			{
				img = PNGEncoder.encode(_bitmapData);
			} else {
				var encoder:JPGEncoder = new JPGEncoder(_quality);
				img = encoder.encode(_bitmapData);
			}
			
			var urlHeader:URLRequestHeader = new URLRequestHeader("Content-type", "application/octet-stream");
			var urlRequest:URLRequest = new URLRequest();
			urlRequest.requestHeaders.push(urlHeader);
			urlRequest.url = _url + "/" + filename + "." + _extension;
			urlRequest.method = URLRequestMethod.POST;
			urlRequest.data = img;
			
			try
			{
				navigateToURL(urlRequest, "_blank");
			} catch (e:Error) {
				trace("Unable to load the URL");
			}
		}
		
		public function timestamp():String
		{
			return String(int((new Date()).valueOf() / 1000));
		}
		
		public function get url():String {
			return _url;
		}
		public function set url(value:String):void {
			_url = value;
		}
		
		public function get bitmapData():BitmapData {
			return _bitmapData;
		}
		public function set bitmapData(value:BitmapData):void {
			_bitmapData = value;
		}
		
		public function get quality():int {
			return _quality;
		}
		public function set quality(value:int):void {
			_quality = value;
		}
	}
	
}